@@ -15,6 +15,7 @@ from __future__ import (division, unicode_literals, print_function,
                         absolute_import)
 
 import contextlib
+import struct
 import time
 import warnings
 
@@ -321,7 +322,7 @@ class MessageBasedResource(Resource):
         :type chunk_size: int
         :param break_on_termchar: Should the reading stop when a termination
             character is encountered.
-        :type brak_on_termchar: bool
+        :type break_on_termchar: bool
 
         :rtype: bytes
 
@@ -449,7 +450,9 @@ class MessageBasedResource(Resource):
         return util.from_ascii_block(block, converter, separator, container)
 
     def read_binary_values(self, datatype='f', is_big_endian=False,
-                           container=list, header_fmt='ieee'):
+                           container=list, header_fmt='ieee',
+                           expect_termination=True, data_points=0,
+                           chunk_size=None):
         """Read values from the device in binary format returning an iterable
         of values.
 
@@ -460,39 +463,72 @@ class MessageBasedResource(Resource):
         :param container: container type to use for the output data.
         :param header_fmt: format of the header prefixing the data. Possible
                            values are: 'ieee', 'hp', 'empty'
+        :param expect_termination: when set to False, the expected length of
+                                   the binary values block does not account
+                                   for the final termination character (the
+                                   read termination)
+        :param data_points: Number of points expected in the block. This is
+                            used only if the instrument does not report it
+                            itself. This will be converted in a number of bytes
+                            based on the datatype.
+        :param chunk_size: Size of the chunks to read from the device. Using
+                           larger chunks may be faster for large amount of
+                           data.
         :returns: the answer from the device.
         :rtype: type(container)
 
         """
-        block = self._read_raw()
-        expected_length = 0
+        block = self._read_raw(chunk_size)
 
         if header_fmt == 'ieee':
             offset, data_length = util.parse_ieee_block_header(block)
-            expected_length = offset + data_length
+
+            # Allow to support instrument such as the Keithley 2000 that do not
+            # report the length of the block
+            data_length = data_length or data_points*struct.calcsize(datatype)
+
         elif header_fmt == 'hp':
             offset, data_length = util.parse_hp_block_header(block,
                                                              is_big_endian)
-            expected_length = offset + data_length
+        elif header == 'empty':
+            offset = 0
+        else:
+            raise ValueError("Invalid header format. Valid options are 'ieee',"
+                             " 'empty', 'hp'")
 
-        if self._read_termination is not None:
+        expected_length = offset + data_length
+
+        if expect_termination and self._read_termination is not None:
             expected_length += len(self._read_termination)
 
-        while len(block) < expected_length:
-            block.extend(self._read_raw())
+        # Read all the data if we know what to expect.
+        if data_length != 0:
+            block.extend(self.read_bytes(expected_length - len(block),
+                                         chunk_size=chunk_size))
+        else:
+            # Backward compatibility. This is dangerous and should probably be
+            # removed
+            msg = ('Reading binary data without a known length is error prone,'
+                   ' and should be avoided. This capability will will be'
+                   ' removed in future versions.\n'
+                   'If the instrument does not report the length of the block '
+                   'as part of the transfer, it may be because the size is '
+                   'fixed or can be accessed from the instrumentin using a '
+                   'specific command. You should find the expected number of '
+                   'bytes and pass it using the `data_length` keyword.')
+            warnings.warn(msg, FutureWarning)
+            # Do not keep reading since we may have already read everything
+
+            # Set the datalength to None to infer it from the block length
+            data_length = None
+
 
         try:
-            if header_fmt == 'ieee':
-                return util.from_ieee_block(block, datatype, is_big_endian,
-                                            container)
-            elif header_fmt == 'hp':
-                return util.from_hp_block(block, datatype, is_big_endian,
+            # Do not reparse the headers since it was already done and since
+            # this allows for custom data length
+            return util.from_binary_block(block, offset, data_length,
+                                          datatype, is_big_endian,
                                           container)
-            elif header_fmt == 'empty':
-                return util.from_binary_block(block, 0, None, datatype,
-                                              is_big_endian, container)
-            else:
-                raise
         except ValueError as e:
             raise errors.InvalidBinaryFormat(e.args)
 
@@ -624,7 +660,9 @@ class MessageBasedResource(Resource):
                                       delay)
 
     def query_binary_values(self, message, datatype='f', is_big_endian=False,
-                            container=list, delay=None, header_fmt='ieee'):
+                            container=list, delay=None, header_fmt='ieee',
+                            expect_termination=True, data_points=0,
+                            chunk_size=None):
         """Query the device for values in binary format returning an iterable
         of values.
 
@@ -636,6 +674,17 @@ class MessageBasedResource(Resource):
         :param container: container type to use for the output data.
         :param delay: delay in seconds between write and read operations.
                       if None, defaults to self.query_delay
+        :param expect_termination: when set to False, the expected length of
+                                   the binary values block does not account
+                                   for the final termination character (the
+                                   read termination)
+        :param data_points: Number of points expected in the block. This is
+                            used only if the instrument does not report it
+                            itself. This will be converted in a number of bytes
+                            based on the datatype.
+        :param chunk_size: Size of the chunks to read from the device. Using
+                           larger chunks may be faster for large amount of
+                           data.
         :returns: the answer from the device.
         :rtype: list
         """
@@ -650,7 +699,8 @@ class MessageBasedResource(Resource):
             time.sleep(delay)
 
         return self.read_binary_values(datatype, is_big_endian, container,
-                                       header_fmt)
+                                       header_fmt, expect_termination,
+                                       data_points, chunk_size)
 
     def ask_for_values(self, message, fmt=None, delay=None):
         """A combination of write(message) and read_values()
@@ -699,6 +749,18 @@ class MessageBasedResource(Resource):
         yield
         self.set_visa_attribute(constants.VI_ATTR_TERMCHAR, term)
 
+    def flush(self, mask):
+        """Manually clears the specified buffers.
+
+        Depending on the value of the mask this can cause the buffer data
+        to be written to the device.
+
+        :param mask: Specifies the action to be taken with flushing the buffer.
+            See highlevel.VisaLibraryBase.flush for a detailed description.
+
+        """
+        self.visalib.flush(self.session, mask)
+
 
 # Rohde and Schwarz Device via Passport. Not sure which Resource should be.
 MessageBasedResource.register(constants.InterfaceType.rsnrp,
